VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "ClassOutputDetailFile"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
' HANDLE DETAIL OUTPUT FILE CLASS

Private Result As ClassProblemResult            ' parent problem result object reference
Private Definition As ClassProblemDefinition    ' parent problem definition
Private fileID As Long                          ' output file handle

Private outputTableau()                         ' output tableau with column widths

Private Const lineLength = 100                  ' line length in characters
Private Const leftMargin = "   "                ' left margin in characters

' show result testing

Public Sub PrintTestResult()
    
    Dim buffer As String
    Dim i As Long, j As Long
    
    Dim tempValue As Double
    
    SingleLine "   R E S U L T   T E S T I N G", "=", 1, , "="
    
    SingleLine ""
    
    For i = 1 To Definition.M
    
        tempValue = 0
        
        For j = 1 To Definition.S
        
            tempValue = tempValue + Definition.A(i, j) * Result.Xj(j)
            
        Next j
        
        buffer = "constraint no. " & Spaces(i, 3) & _
                 IIf(Result.ConstraintTest(i), " TRUE : ", " FALSE: ") & _
                 GiveFormatted(tempValue) & " " & _
                 Definition.ConstraintType(i) & "= " & _
                 GiveFormatted(Definition.RHS(i))
        
        SingleLine buffer
        
    Next i
    
    SingleLine "Test result: " & Result.TestResult, , 1, 1

End Sub

' summary for optimal solution

Public Sub PrintSummary()
                        
    Dim buffer As String
    Dim j As Long
    
    SingleLine "   S U M M A R Y ", "=", 1, , "="
    
    SingleLine "Status: " & Result.StatusName, , 1
    
    SingleLine "Degeneracy: " & Result.Degeneracy, , 1
    
    SingleLine "Test to constraints: " & Result.TestResult, , 1, 1
    
    SingleLine Definition.Objective & ": " & GiveFormatted(Result.FunctionValue)
    
    SingleLine "BVS: [" & ArrayToString(Result.BVSAsArray, ", ") & " ]", , 1, 1
    
    For j = 1 To Definition.S
        
        SingleLine "x(" & j & ") = " & GiveFormatted(Result.Xj(j))
    
    Next j
    
    SingleLine "Number of iterations: " & Result.Counters.Counter(cntIterations), , 1
    SingleLine "Number of additions/subtractions: " & Result.Counters.Counter(cntAddSubs)
    SingleLine "Number of multiplications/divisions: " & Result.Counters.Counter(cntMultDivs)
    SingleLine "Number of loops: " & Result.Counters.Counter(cntLoops)
    SingleLine "Number of decisions: " & Result.Counters.Counter(cntDecis)
    SingleLine " ", , 1, 1, "="

End Sub

'
'   initiation of detail output file object
'
Public Sub InitialDeclarations(newResult As ClassProblemResult)

    Set Result = newResult
    
    Set Definition = newResult.Definition

End Sub

' open output detail file

Public Sub OpenOutputFile()
    
On Error GoTo errorOpenOutputFile

    fileID = OpenFile(Result.DetailFile(returnFileFull), openForOutput)
    
    Exit Sub
    
errorOpenOutputFile:

    If fileID > 0 Then Close #fileID
    
    Err.Raise Err.Number, , "Open method output file error." & _
                            NewLine(Err.Description)
    
End Sub

' close output detail file

Public Sub CloseOutputFile()
    
    Close #fileID
    
End Sub

' print single line to file

Public Sub SingleLine(lineText As String, _
                      Optional lineOfCharactersBefore As String = "", _
                      Optional emptyLinesBefore As Long = 0, _
                      Optional emptylinesAfter As Long = 0, _
                      Optional lineOfCharactersAfter = "")
    
    Dim i As Long
    
    If lineOfCharactersBefore <> "" Then
    
        Print #fileID, String(lineLength, lineOfCharactersBefore)
        
    End If
    
    For i = 1 To emptyLinesBefore
        
        Print #fileID, ""
    
    Next i
        
    Print #fileID, leftMargin & lineText
    
    For i = 1 To emptylinesAfter
    
        Print #fileID, ""
    
    Next i
    
    If lineOfCharactersAfter <> "" Then
    
        Print #fileID, String(lineLength, lineOfCharactersAfter)
        
    End If

End Sub

' print result detail header

Public Sub ResultDetailHeader()
    
    SingleLine "   " & Stuff(Result.MethodName, "   "), "=", 1, 1, "="
    
    SingleLine "Problem source    : " & Definition.Source
    
    SingleLine "Comment           : " & Definition.Comment
    
    SingleLine "Execution started : " & Result.StartedWhen
    
    SingleLine "Problem Definition: ", , 1
    
    ProblemDescription

End Sub

' print problem description

Public Sub ProblemDescription()

    Dim buffer As String: buffer = ""
    Dim i As Long, j As Long
    
    With Definition
    
        buffer = GiveFormatted(.Cj(1)) & "*x(1)"
        
        For j = 2 To .S
        
            buffer = buffer & " + " & GiveFormatted(.Cj(j)) & "*x(" & j & ")"
    
        Next j
        
        SingleLine "   " & .Objective & ": " & buffer, , 1, 1
        
        For i = 1 To .M
        
            buffer = GiveFormatted(.A(i, 1)) & "*x(1)"
            
            For j = 2 To .S
 
                buffer = buffer & " + " & GiveFormatted(.A(i, j)) & "*x(" & j & ")"
            
            Next j
        
            buffer = buffer & " " & .ConstraintType(i) & "= " & GiveFormatted(.RHS(i))
            
            SingleLine IIf(i = 1, "Constr.: ", "         ") & buffer
        
        Next i

    End With

End Sub

' print tableau

Public Sub PrintTableau(tableau, _
                        S As Long, _
                        D As Long, _
                        U As Long, _
                        M As Long, _
                        BVS)
    
    Dim i As Long, j As Long
    Dim buffer As String
    
    ' header
    ReDim outputTableau(-1 To UBound(tableau, 1), -1 To UBound(tableau, 2))
    
    ' set initial column widths
    For j = LBound(outputTableau) To UBound(outputTableau)
        
        outputTableau(-1, j) = 0
    
    Next j
    
    SetField 0, -1, "i"
    
    SetField 0, 0, "BVS"
    
    ' indicate surplus/slack or artificial variable
    For j = 1 To UBound(tableau, 2)
        
        SetField 0, j, j & IIf(j <= S, "", IIf(j <= S + D, "S", "A"))
    
    Next j
    
    SetField 0, UBound(tableau, 2), "RHS"
    
    ' data
    For i = 1 To M + 1
    
        ' first two columns (row-header)
        If i <= M Then
            
            SetField i, -1, i
            SetField i, 0, BVS(i)
        
        Else
            
            ' last row
            SetField i, -1, "Zj"
            SetField i, 0, ""
        
        End If
        
        ' values
        For j = 1 To UBound(tableau, 2)
        
            SetField i, j, tableau(i, j)
        
        Next j
        
    Next i
    
    ' do printing
    Print #fileID, ""
    
    For i = LBound(outputTableau, 1) + 1 To UBound(outputTableau, 1)
        
        buffer = " "
        
        For j = LBound(outputTableau, 2) To UBound(outputTableau, 2)
            
            buffer = buffer & _
                     Right(Space(outputTableau(-1, j)) & _
                     outputTableau(i, j), outputTableau(-1, j)) & _
                     IIf(j < UBound(outputTableau, 2), " | ", " ")
        
        Next j
        
        Print #fileID, buffer
        
        ' top and bottom line
        If ((i = 0) Or (i = M)) Then
        
            buffer = "-"
            
            For j = LBound(outputTableau, 2) To UBound(outputTableau, 2)
                
                buffer = buffer & String(outputTableau(-1, j), "-") & _
                         IIf(j < UBound(outputTableau, 2), "-+-", "-")
            
            Next j
            
            Print #fileID, buffer
        
        End If
        
    Next i
    
    Print #fileID, ""

End Sub

' set field formatted and calculate new column width if necessery

Private Sub SetField(r As Long, _
                     c As Long, _
                     fieldValue)

    outputTableau(r, c) = GiveFormatted(fieldValue)
    
    ' set new column width in characters
    If outputTableau(-1, c) < Len(outputTableau(r, c)) Then
    
        outputTableau(-1, c) = Len(outputTableau(r, c))
        
    End If

End Sub

'
'   Initiate
'
Private Sub Class_Initialize()
'   initialization
End Sub

'
'   Terminate
'
Private Sub Class_Terminate()

    Set Result = Nothing
    Set Definition = Nothing
End Sub
