VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "ClassProblem"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False

'   CLASS MODULE: PROBLEM OBJECT

'   fields containing object data

Private Type FieldType

    Definition As New ClassProblemDefinition

    arrayResults(1 To 2) As ClassProblemResult

    ProblemSaved As Boolean

    DisplayGlobals As New Collection
    
    DisplayDefinition As MyGrid

    DisplayResultSummary As MyGrid

    DisplayResultDetails As MyRichTextBox
    
    NoDisplay As Boolean
    
    IsInAnalysis As Boolean
    
    AnalysisRowIndex As Long

End Type

Private fields As FieldType

Public Event SaveStateChanged(Saved As Boolean)

'
'   row index within analysis
'
Public Property Get AnalysisRowIndex() As Long

    AnalysisRowIndex = fields.AnalysisRowIndex

End Property

'
'   row index within analysis
'
Public Property Let AnalysisRowIndex(newRowIndex As Long)

    fields.AnalysisRowIndex = newRowIndex

End Property


'Private WithEvents fieldDisplayDefinition As MyGrid
'
'   property get Definition
'
Public Property Get Definition() As ClassProblemDefinition

    Set Definition = fields.Definition

End Property

'
'   property set Definition
'
Public Property Set Definition(newDefinitionObject As ClassProblemDefinition)
    
    Set fields.Definition = newDefinitionObject
    
    fields.Definition.InitialDeclarations Me

End Property

Public Sub DisplayInAnalysis(mgAnalysis As MyGrid, _
                             Optional rowIndex As Long = -1)
                             
    With mgAnalysis
    
        Dim buffer() As String: ReDim buffer(1 To CLng(.OtherData("last column")))
            
        buffer(1) = "<" & ProblemName
        buffer(2) = Definition.Objective
        buffer(3) = Definition.M
        buffer(4) = Definition.S
        buffer(5) = "." & Definition.ConstraintTypeAsStrArray
                
        Dim method As EnumMethod
        Dim startCol As Long
        
        For method = PushPull To Simplex
        
            startCol = .OtherData(EnumMethodAsString(method))
        
            buffer(startCol) = Result(method).StatusName
            
            If Result(method).Status > NotCalculated Then
                
                buffer(startCol + 1) = Result(method).Degeneracy
                
                buffer(startCol + 2) = Result(method).FunctionValue
                
                buffer(startCol + 3) = ArrayToString(Result(method).BVSAsArray)
                
                buffer(startCol + 4) = Result(method).Counters(cntIterations)
                buffer(startCol + 5) = Result(method).Counters(cntAddSubs)
                buffer(startCol + 6) = Result(method).Counters(cntMultDivs)
                buffer(startCol + 7) = Result(method).Counters(cntLoops)
                buffer(startCol + 8) = Result(method).Counters(cntDecis)
                
            End If
        
        Next method
            
        If rowIndex > -1 Then AnalysisRowIndex = rowIndex
            
        .SetValueRow buffer, AnalysisRowIndex
            
    End With
                             
                             
End Sub

'
'   property get IsInAnalysis
'
Public Property Get IsInAnalysis() As Boolean

    IsInAnalysis = fields.IsInAnalysis

End Property

'
'   property let IsInAnalysis
'
Public Property Let IsInAnalysis(newValue As Boolean)

    fields.IsInAnalysis = newValue

End Property


'
'   property let NoDisplay
'
Public Property Let NoDisplay(newValue As Boolean)

    fields.NoDisplay = newValue

End Property

'
'   property get NoDisplay
'
Public Property Get NoDisplay() As Boolean

    NoDisplay = fields.NoDisplay

End Property


'
'   refresh and redisplay definition and results
'
Public Sub ShowFromAnalysis(newDisplayGlobals As Collection, _
                            newDisplayDefinition As MyGrid, _
                            newDisplayResultSummary As MyGrid, _
                            newDisplayResultDetails As MyRichTextBox)

    '   determine NoDisplayOption

    NoDisplay = False

    IsInAnalysis = True

    '   set references to display controls

    Set DisplayGlobals = newDisplayGlobals

    Set DisplayDefinition = newDisplayDefinition

    Set DisplayResultSummary = newDisplayResultSummary

    Set DisplayResultDetails = newDisplayResultDetails

    '   show definition

    Set Definition.DisplayDefinition = newDisplayDefinition

    Definition.ShowDefinition

    '   initiate result objects

    Result(PushPull).ShowResult

    Result(Simplex).ShowResult

    CalcDifference

End Sub

'
'   property get Result
'
Public Property Get Result(method As EnumMethod) As ClassProblemResult

    Set Result = fields.arrayResults(method)

End Property

'
'   property Result
'   set
Public Property Set Result(method As EnumMethod, _
                           newResultObject As ClassProblemResult)

    Set fields.arrayResults(method) = newResultObject
    
    fields.arrayResults(method).InitialDeclarations Me, method

End Property

'
'   property get Problem Saved status
'
Public Property Get ProblemSaved() As Boolean

    ProblemSaved = fields.ProblemSaved

End Property

'
'   property let Problem Saved status
'
Public Property Let ProblemSaved(newValue As Boolean)

    fields.ProblemSaved = newValue

    RaiseEvent SaveStateChanged(newValue)

End Property

'
'   property get Name
'
Public Property Get ProblemName() As String
    
    ProblemName = Definition.ProblemName

End Property

'
'   property get DisplayGlobals
'
Public Property Get DisplayGlobals(Optional key As String)

    Set DisplayGlobals = fields.DisplayGlobals(key)

End Property

'
'   property set DisplayGlobals
'
Public Property Set DisplayGlobals(Optional key As String, _
                                   newObject)

    Set fields.DisplayGlobals = newObject

End Property

'
'   property get DisplayDefinition
'
Public Property Get DisplayDefinition() As MyGrid

    Set DisplayDefinition = fields.DisplayDefinition
    
'    With fields.DisplayDefinition
    
'        .EditEnabled = True
    
'        Dim titleMatrix() As String
'        ReDim titleMatrix(0, 0): titleMatrix(0, 0) = ".MAX"
    
'        .SetHeaderTitle titleMatrix

'    End With

End Property

'
'   property set DisplayDefinition
'
Public Property Set DisplayDefinition(newObject As MyGrid)

    Set fields.DisplayDefinition = newObject

    If NoDisplay Then Exit Property
    
    With fields.DisplayDefinition
    
        .EditEnabled = True
    
        Dim titleMatrix() As String
        ReDim titleMatrix(0, 0): titleMatrix(0, 0) = ".MAX"
    
        .SetHeaderTitle titleMatrix

    End With

End Property

'
'   property get DisplayResultSummary
'
Public Property Get DisplayResultSummary() As MyGrid

    Set DisplayResultSummary = fields.DisplayResultSummary

End Property

'
'   property set DisplayResultSummary
'
Public Property Set DisplayResultSummary(newObject As MyGrid)

    Set fields.DisplayResultSummary = newObject

    If NoDisplay Then Exit Property
    
    Dim titleMatrix(0, 1): SetMatrixRowFromArray 0, Array(". ", "#-"), titleMatrix
    
    With fields.DisplayResultSummary
    
        .showSelectedRow = True
        
        .SetHeaderTitle titleMatrix
    
        .SetHeaderColumn Array(EnumMethodAsString(PushPull)), PushPull
        .SetHeaderColumn Array(EnumMethodAsString(Simplex)), Simplex
        .SetHeaderColumn Array("DIFFERENCE"), Simplex + 1

        .SetHeaderRow Array(">Status: ", "#-"), 1
        .SetHeaderRow Array(">Objective Function Value: ", "#-")
        .SetHeaderRow Array(">Degeneracy: ", "#-")
        .SetHeaderRow Array(">Total of Iterations: ", "#-")
        .SetHeaderRow Array(">Total of Adds/Subs: ", "#-")
        .SetHeaderRow Array(">Total of Mult/Divs: ", "#-")
        .SetHeaderRow Array(">Total of Loops: ", "#-")
        .SetHeaderRow Array(">Total of Decisions: ", "#-")
        .SetHeaderRow Array(">Testing to Constraints: ", "#-")
'        .SetHeaderRow Array(">Detail File Name: ", "#-")
'        .SetHeaderRow Array(">Execution Started ", "<Date: ")
'        .SetHeaderRow Array("#|", "<Time: ")
'
'        .OtherData("BVS start") = 8
'
'        .OtherData("X(j) start") = 8

        .OtherData("BVS start") = 9 '5

        .OtherData("X(j) start") = 9 '5
    
    End With

End Property

'
'   property get DisplayResultDetails
'
Public Property Get DisplayResultDetails() As MyRichTextBox

    Set DisplayResultDetails = fields.DisplayResultDetails

End Property

'
'   property set DisplayResultDetails
'
Public Property Set DisplayResultDetails(newObject As MyRichTextBox)

    Set fields.DisplayResultDetails = newObject
    
    If NoDisplay Then Exit Property
    
    With fields.DisplayResultDetails
    
        .SetBoxName PushPull, EnumMethodAsString(PushPull)
        
        .SetBoxName Simplex, EnumMethodAsString(Simplex)

    End With

End Property

'
'   initiate linear programming problem object
'
Public Sub InitialDeclarations(newDisplayGlobals As Collection, _
                               newDisplayDefinition As MyGrid, _
                               newDisplayResultSummary As MyGrid, _
                               newDisplayResultDetails As MyRichTextBox)
    
    '   determine NoDisplayOption
    
    NoDisplay = False
    
    '   set references to display controls
    
    Set DisplayGlobals = newDisplayGlobals
    
    Set DisplayDefinition = newDisplayDefinition
    
    Set DisplayResultSummary = newDisplayResultSummary
    
    Set DisplayResultDetails = newDisplayResultDetails
    
    '   initiate definition object
    
    Set Definition = New ClassProblemDefinition
    
    '   initiate result objects
    
    Set Result(PushPull) = New ClassProblemResult
    
    Set Result(Simplex) = New ClassProblemResult
    
End Sub

'
'   initiate linear programming problem object
'
Public Sub InitialWithNoDisplay()
    
    '   determine NoDisplayOption
    
    NoDisplay = True
    
    '   set references to display controls
    
    Set DisplayGlobals = Nothing
    
    Set DisplayDefinition = Nothing
    
    Set DisplayResultSummary = Nothing
    
    Set DisplayResultDetails = Nothing
    
    '   initiate definition object
    
    Set Definition = New ClassProblemDefinition
    
    '   initiate result objects
    
    Set Result(PushPull) = New ClassProblemResult
    
    Set Result(Simplex) = New ClassProblemResult
    
End Sub


'
'   create new problem
'
Public Sub CreateNew(NewProblemName As String)
    
    Definition.ResetToDefault NewProblemName
    
    Result(PushPull).ResetToDefault
    
    Result(Simplex).ResetToDefault
    
'    ProblemSaved = True

End Sub

'
'   open problem defined in selected file
'
Public Sub OpenProblem(problemFileName As String)

On Error GoTo errorOpenProblem

    Definition.ReadFromFile problemFileName
    
    Result(PushPull).ReadResultData
    
    Result(Simplex).ReadResultData
    
    ProblemSaved = True
    
    CalcDifference
    
    Exit Sub

errorOpenProblem:

    Err.Raise Err.Number, , "File: " & problemFileName & _
                            NewLine(Err.Description)
    
End Sub

'
'   import problem data from selected file
'
Public Sub ImportProblem(problemFileName As String)
    
On Error GoTo errorImportProblem
    
    Definition.ImportFromFile problemFileName
    
    Result(PushPull).ResetToDefault
    
    Result(Simplex).ResetToDefault
    
    ProblemSaved = False
    
    Exit Sub

errorImportProblem:

    Err.Raise Err.Number, , "File: " & problemFileName & _
              NewLine(Err.Description)
    
End Sub

'
'   save problem definition and results
'
Public Sub SaveProblem(Optional newDefinitionFile As String = "")

On Error GoTo errorSaveProblem

    If newDefinitionFile = "" Then newDefinitionFile = Definition.DefinitionFile

    Dim fileID As Long: fileID = OpenFile(newDefinitionFile, openForOutput)
    
    Definition.DefinitionFile = newDefinitionFile

    Definition.SaveDefinition fileID

    Result(PushPull).SaveResultData fileID
    
    Result(Simplex).SaveResultData fileID

    ProblemSaved = True
    
    Close #fileID
    
    Exit Sub
    
errorSaveProblem:
    
    If fileID > 0 Then Close #fileID

    Err.Raise Err.Number, , Err.Description & _
                            NewLine("File: " & newDefinitionFile)

End Sub


'
'   calculate difference
'
Public Sub CalcDifference()
    
    If NoDisplay Then Exit Sub
    
    DisplayResultSummary.ClearColumnValues DisplayResultSummary.ValueColCount
    
    Dim Res(0 To 1) As ClassProblemResult
    Set Res(0) = Result(PushPull)
    Set Res(1) = Result(Simplex)
    
    If Res(0).Status = NotCalculated Or Res(1).Status = NotCalculated Then Exit Sub
    
    Dim c As Long: c = DisplayResultSummary.ValueColCount
    
    Dim D As Double
    With DisplayResultSummary

        .ClearColumnValues .ValueColCount
        
        .SetValueSingle IIf(Res(0).Status <> Res(1).Status, ".< >", "= ="), 1, c
        
        D = Res(0).FunctionValue - Res(1).FunctionValue
        .SetValueSingle IIf(D = 0, "= =", GiveFormatted(D)), , c
        
        .SetValueSingle IIf(Res(0).Degeneracy <> Res(1).Degeneracy, ".< >", "= ="), , c
        
        D = Res(0).Counters(cntIterations) - Res(1).Counters(cntIterations)
        .SetValueSingle IIf(D = 0, "= =", GiveFormatted(D)), , c

        D = Res(0).Counters(cntAddSubs) - Res(1).Counters(cntAddSubs)
        .SetValueSingle IIf(D = 0, "= =", GiveFormatted(D)), , c
        
        D = Res(0).Counters(cntMultDivs) - Res(1).Counters(cntMultDivs)
        .SetValueSingle IIf(D = 0, "= =", GiveFormatted(D)), , c

        D = Res(0).Counters(cntLoops) - Res(1).Counters(cntLoops)
        .SetValueSingle IIf(D = 0, "= =", GiveFormatted(D)), , c

        D = Res(0).Counters(cntDecis) - Res(1).Counters(cntDecis)
        .SetValueSingle IIf(D = 0, "= =", GiveFormatted(D)), , c

        .SetValueSingle IIf(Res(0).TestResult <> Res(1).TestResult, ".< >", "= ="), , c

        Dim i As Long

        For i = 1 To Definition.M

            D = Res(0).BVS(i) - Res(1).BVS(i)
            .SetValueSingle IIf(D = 0, "= =", GiveFormatted(D)), , c
        Next i

        For i = 1 To Definition.S

            D = Res(0).Xj(i) - Res(1).Xj(i)
            .SetValueSingle IIf(D = 0, "= =", GiveFormatted(D)), , c
        Next i

    End With
    
    Set Res(0) = Nothing
    Set Res(1) = Nothing
End Sub

'
'   initializing of the class
'
Private Sub Class_Initialize()

'   no special initializations
End Sub

'
'   terminating the class
'
Private Sub Class_Terminate()

    With fields
    
        Dim i As Long
        For i = LBound(.arrayResults) To UBound(.arrayResults)
    
            Set .arrayResults(i) = Nothing
        Next i
    
        Set .DisplayDefinition = Nothing

        Set .DisplayResultSummary = Nothing

        Set .DisplayResultDetails = Nothing
    
        Set .Definition = Nothing
        
    End With

End Sub

