VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "ClassProblemDefinition"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False

'   CLASS MODULE: PROBLEM DEFINITION

'   default values for problem definition

Private Const defaultDefinitionFile = "NewProblem.txt"
Private Const defaultSource = "Unknown source."
Private Const defaultComment = "No comment."
Private Const defaultVariableNo = 3
Private Const defaultConstraintNo = 3
Private Const defaultObjective = "MAX"
Private Const defaultConstraintType = "<"
Private Const defaultAij As Double = 0
Private Const defaultCj As Double = 0
Private Const defaultRHS As Double = 0

'   fields containing object data

Private Type FieldType

    parentProblem As ClassProblem
    
    DefinitionFile As String
    Source As String
    Comment As String
    
    M As Long
    S As Long
    Objective As String
    arrayCj() As Double
    arrayRHS() As Double
    matrixA() As Double
    arrayConstraintType() As String

End Type

Private fields As FieldType

Public WithEvents DisplayDefinition As MyGrid
Attribute DisplayDefinition.VB_VarHelpID = -1

'
'   property get Constraint Type Array as Delimited String
'
Public Property Get ConstraintTypeAsStrArray() As String

    ConstraintTypeAsStrArray = ArrayToString(fields.arrayConstraintType)
''''''''''''''''''''''''''''''''''''''''''''''
'''' I can name property let also! and for other arrays an matrix too !!!!!!!!!!!
End Property

'
'   property get noDisplay
'
Public Property Get NoDisplay() As Boolean

    NoDisplay = fields.parentProblem.NoDisplay

End Property

'
'   show definition in display
'
Public Sub ShowDefinition()

    DisplayGlobals("Objective") = Objective
    
    M(True) = fields.M
    
    S(True) = fields.S
    
    DisplayGlobals("Name") = ProblemName
    
    DisplayGlobals("Source") = Source
    
    DisplayGlobals("Comment") = Comment
    

End Sub

'
'   initializing of the class
'
Private Sub Class_Initialize()

    fields.M = 1
    fields.S = 1
    
    ReDim fields.matrixA(1 To fields.M, 1 To fields.M): fields.matrixA(1, 1) = defaultAij
    ReDim fields.arrayCj(1 To fields.S): fields.arrayCj(1) = defaultCj
    ReDim fields.arrayConstraintType(1 To fields.M): fields.arrayConstraintType(1) = defaultConstraintType
    ReDim fields.arrayRHS(1 To fields.M): fields.arrayRHS(1) = defaultRHS

End Sub

'
'   terminating the class
'
Private Sub Class_Terminate()

    Set fields.parentProblem = Nothing
    Set DisplayDefinition = Nothing
    
End Sub


'
'   property get DisplayGlobals
'
Private Property Get DisplayGlobals(key As String) As Control

    Set DisplayGlobals = fields.parentProblem.DisplayGlobals(key)

End Property

'
'   property get DisplayResultSummary
'
Private Property Get DisplayResultSummary() As MyGrid

    Set DisplayResultSummary = fields.parentProblem.DisplayResultSummary

End Property


'
'   property get ProblemName
'
Public Property Get ProblemName() As String
    
    ProblemName = ReturnFileAs(fields.DefinitionFile, returnNameInFile)

End Property

'
'   property get Definition file
'
Public Property Get DefinitionFile(Optional getFileAs As _
                                            EnumReturnFileNameStyle = returnFileFull) _
                                   As String
    
    DefinitionFile = ModuleUtilities.ReturnFileAs(fields.DefinitionFile, getFileAs)
    
End Property

'
'   property let Definition file
'
Public Property Let DefinitionFile(Optional getFileAs As _
                                            EnumReturnFileNameStyle = returnFileFull, _
                                   newValue As String)

    fields.DefinitionFile = newValue

    If Not NoDisplay Then DisplayGlobals("Name") = ProblemName

End Property

'
'   property get problem saved status
'
Public Property Get ProblemSaved() As Boolean

    ProblemSaved = fields.parentProblem.ProblemSaved

End Property

'
'   property let problem saved status
'
Public Property Let ProblemSaved(newValue As Boolean)

    fields.parentProblem.ProblemSaved = newValue

End Property

'
'   property get Source
'
Public Property Get Source() As String

    Source = fields.Source

End Property

'
'   property let Source
'
Public Property Let Source(newValue As String)
    
    fields.Source = newValue

    If Not NoDisplay Then DisplayGlobals("Source") = newValue

End Property

'
'   property get Comment
'
Public Property Get Comment() As String

    Comment = fields.Comment

End Property

'
'   property let Comment
'
Public Property Let Comment(newValue As String)
    
    fields.Comment = newValue

    If Not NoDisplay Then DisplayGlobals("Comment") = newValue

End Property

'
'   property get m: number of constraints
'
Public Property Get M(Optional Reset As Boolean = False) As Long
    
    M = fields.M

End Property

'
'   property let M: number of constraints
'
Public Property Let M(Optional Reset As Boolean = False, _
                      newValue As Long)
    
    If Not IsNumeric(newValue) Then
    
        Err.Raise vbObjectError + ModuleUtilities.errorNumberBadData, , _
                  "M is not numeric: " & newValue
        
    End If

    If (Not Reset) And (newValue = fields.M) Then Exit Property
    
    Dim oldM As Long: oldM = IIf(Reset, 0, fields.M)
    
    fields.M = CLng(newValue)
    
    If Not Reset Then
    
        ModuleUtilities.ReDimMatrix fields.matrixA, 1, fields.M, 1, fields.S, defaultAij
        ModuleUtilities.ReDimArray fields.arrayRHS, 1, fields.M, defaultRHS
        ModuleUtilities.ReDimArray fields.arrayConstraintType, 1, fields.M, defaultConstraintType

    End If
    
    If NoDisplay Then Exit Property
    
    DisplayGlobals("M") = fields.M
    
    '   reset display definition
    
    Dim i As Long
    
    If oldM < fields.M Then
    
        DisplayDefinition.FocusToZero
        
        For i = oldM + 1 To fields.M
        
            DisplayDefinition.SetHeaderRow Array(i), i
            DisplayDefinition.SetValueRow MergeArrays(ModuleUtilities.GetMatrixRowAsArray(i, fields.matrixA), _
                MergeArrays(Array("." & ConstraintType(i)), Array(RHS(i)))), i
            
        Next i
    
    End If
        
    DisplayDefinition.SetHeaderRow Array("C(j)"), fields.M + 1, True
    DisplayDefinition.SetValueRow fields.arrayCj, fields.M + 1
    
    '   reset display results summary
        
    With DisplayResultSummary

        Dim tempArray(): ReDim tempArray(1)
    
        For i = 1 To fields.M
        
            tempArray(0) = IIf(i = 1, ">Basic Variable Set: ", "#|")
            tempArray(1) = i
            
            .SetHeaderRow tempArray, i + .OtherData("BVS start")
        
        Next i
    
        .OtherData("X(j) start") = .OtherData("BVS start") + fields.M
    
        For i = 1 To fields.S
    
            tempArray(0) = IIf(i = 1, ">Variable Values: ", "#|")
            tempArray(1) = "x(" & i & ")"
            
            .SetHeaderRow tempArray, i + .OtherData("X(j) start")
        
        Next i

    End With
    
End Property

'
'   property get S: number of variables
'
Public Property Get S(Optional Reset As Boolean = False) As Long
    
    S = fields.S

End Property

'
'   property let S: number of variables
'
Public Property Let S(Optional Reset As Boolean = False, _
                      newValue As Long)
    
    If Not IsNumeric(newValue) Then
    
        Err.Raise vbObjectError + errorNumberBadData, , _
                  "S is not numeric: " & newValue
        
    End If

    If (Not Reset) And (newValue = fields.S) Then Exit Property
    
    Dim oldS As Long: oldS = IIf(Reset, 0, fields.S)
    
    fields.S = CLng(newValue)

    If Not Reset Then
    
        ModuleUtilities.ReDimMatrix fields.matrixA, 1, fields.M, 1, fields.S, defaultAij
        ModuleUtilities.ReDimArray fields.arrayCj, 1, fields.S, defaultCj
    
    End If
    
    If NoDisplay Then Exit Property
    
    DisplayGlobals("S") = fields.S
    
    '   reset DisplayDefinition
    
    If oldS < fields.S Then
    
        Dim j As Long
        
        DisplayDefinition.FocusToZero
        
        For j = oldS + 1 To fields.S
        
            DisplayDefinition.SetHeaderColumn Array("X(" & j & ")"), j
            DisplayDefinition.SetValueColumn MergeArrays(ModuleUtilities.GetMatrixColumnAsArray(j, fields.matrixA), _
                Array(Cj(j))), j
        
        Next j
    
    End If
        
    DisplayDefinition.SetHeaderColumn Array("     "), fields.S + 1, True
    DisplayDefinition.SetValueColumn fields.arrayConstraintType, fields.S + 1
    
    DisplayDefinition.SetHeaderColumn Array("RHS"), fields.S + 2
    DisplayDefinition.SetValueColumn fields.arrayRHS, fields.S + 2

    '   reset display results summary
        
    With DisplayResultSummary

        Dim tempArray(): ReDim tempArray(1)
    
        Dim i As Long
        For i = 1 To fields.S
    
            tempArray(0) = IIf(i = 1, ">Variable Values: ", "#|")
            tempArray(1) = "x(" & i & ")"
            
            .SetHeaderRow tempArray, i + .OtherData("X(j) start")
        
        Next i

    End With

End Property

'
'   property get Objective: MIN or MAX
'
Public Property Get Objective() As String
    
    Objective = fields.Objective

End Property

'
'   property let Objective: MIN or MAX
'
Public Property Let Objective(newValue As String)
    
    newValue = UCase(Trim(newValue))
    
    If (InStr("MIN MAX", newValue) = 0) Or _
       (newValue = "") Then
        
        Err.Raise vbObjectError + errorNumberBadData, , _
                  "Wrong objective (MIN, MAX): " & newValue & "!"
    
    End If
    
    fields.Objective = newValue
    
    If NoDisplay Then Exit Property
    
    DisplayGlobals("Objective") = newValue

    Dim titleMatrix(0, 0): titleMatrix(0, 0) = newValue
    DisplayDefinition.SetHeaderTitle titleMatrix

End Property

'
'   property get Constraint type: <, =, >
'
Public Property Get ConstraintType(i As Long) As String
    
    ConstraintType = fields.arrayConstraintType(i)

End Property

'
'   property let Constraint type: <, =, >
'
Public Property Let ConstraintType(i As Long, _
                                   newValue As String)
    
    
    If InStr("< = >", newValue) = 0 Then
        
        Err.Raise vbObjectError + errorNumberBadData, , _
                 "Wrong constraint type (<, =, >): " & newValue & "!"
    
    End If
    
    fields.arrayConstraintType(i) = Trim(newValue)

    If Not NoDisplay Then
            
        DisplayDefinition.SetValueSingle "." & GiveFormatted(newValue), i, fields.S + 1

    End If
        
End Property

'
'   property get RHS: right hand side values
'
Public Property Get RHS(i As Long) As Double
    
    RHS = fields.arrayRHS(i)

End Property

'
'   property let RHS: right hand side values
'
Public Property Let RHS(i As Long, newValue As Double)
    
    If Not IsNumeric(newValue) Then
    
        Err.Raise vbObjectError + errorNumberBadData, , _
                  "RHS(" & i & ") is not numeric: " & newValue
        
    End If
    
    fields.arrayRHS(i) = newValue

    If NoDisplay Then Exit Property
    
    DisplayDefinition.SetValueSingle GiveFormatted(newValue), i, fields.S + 2

End Property

'
'   property get Cj: objective function coefficient value
'
Public Property Get Cj(ByVal j As Long) As Double

    Cj = fields.arrayCj(j)

End Property

'
'   property let Cj: objective function coefficient value
'
Public Property Let Cj(ByVal j As Long, _
                       newValue As Double)

    If Not IsNumeric(newValue) Then
    
        Err.Raise vbObjectError + errorNumberBadData, , _
                  "C(" & j & ") is not numeric: " & newValue
        
    End If
    
    fields.arrayCj(j) = newValue
    
    If NoDisplay Then Exit Property
    
    DisplayDefinition.SetValueSingle GiveFormatted(newValue), fields.M + 1, j

End Property

'
'   property get A(i, j): matrix with constraint functions coefficients
'
Public Property Get A(ByVal i As Long, _
                      ByVal j As Long) As Double
    
    A = fields.matrixA(i, j)

End Property

'
'   property let A(i, j): matrix with constraint functions coefficients
'
Public Property Let A(ByVal i As Long, _
                      ByVal j As Long, _
                      newValue As Double)
    
    If Not IsNumeric(newValue) Then
    
        Err.Raise vbObjectError + errorNumberBadData, , _
                  "A(" & i & ", " & j & ") is not numeric: " & newValue
        
    End If
    
    fields.matrixA(i, j) = newValue
    
    If NoDisplay Then Exit Property
    
    DisplayDefinition.SetValueSingle GiveFormatted(newValue), i, j

End Property

'
'   initiate linear programming problem object
'
Public Sub InitialDeclarations(newParentProblem As ClassProblem)
    
    Set fields.parentProblem = newParentProblem
    
    Set DisplayDefinition = fields.parentProblem.DisplayDefinition
    
End Sub


'
' create new problem
'
Public Sub ResetToDefault(NewProblemName As String)
    
'    DefinitionFile = lastSaveProblemDir & "\" & defaultDefinitionFile
    DefinitionFile = RepositoryFolder & "\" & NewProblemName & ".txt"
        
    Source = defaultSource
    
    Comment = defaultComment
    
    ProblemSaved = False
    
    ReDimMatrix fields.matrixA, 1, defaultConstraintNo, 1, defaultVariableNo, defaultAij, False
    ReDimArray fields.arrayCj, 1, defaultVariableNo, defaultCj, False
    ReDimArray fields.arrayRHS, 1, defaultConstraintNo, defaultRHS, False
    ReDimArray fields.arrayConstraintType, 1, defaultConstraintNo, defaultConstraintType, False
    
    M(True) = defaultConstraintNo
    S(True) = defaultVariableNo
    
    Objective = defaultObjective
    
End Sub


'
'   read data from definition file
'
Public Sub ReadFromFile(problemFileName As String)
    
    Dim fileID As Long: fileID = OpenFile(problemFileName, openForInput)
    
On Error GoTo errorReadFromFile
    
    Dim success As Long: success = 0
    
    Dim parameterValue As String
    
    Do While EOF(fileID) = False
        
        Select Case GetParameter(fileID, parameterValue)
            
        Case "source": Source = parameterValue: success = success + 1
        
        Case "comment": Comment = parameterValue: success = success + 1

        Case "objective": Objective = parameterValue: success = success + 1
            
        Case "m": M = CLng(parameterValue): success = success + 1
            
        Case "s": S = CLng(parameterValue): success = success + 1
            
        Case "c(j)": ReadArrayCj parameterValue: success = success + 1
            
        Case "constraints": ReadConstraints fileID: success = success + 1
        
        End Select
    
    Loop
    
    Close #fileID
    
    If success < 7 Then
    
        Err.Raise vbObjectError + errorNumberBadData, , _
                  "Unknown or missing definition problem data structure."
    
    End If
    
    DefinitionFile = problemFileName
    
    Exit Sub

errorReadFromFile:
    
    If fileID > 0 Then Close #fileID
    
    Err.Raise Err.Number, , "Error reading problem definition file!" & _
                            NewLine(Err.Description)

End Sub



'
'   read whole Cj array
'
Private Sub ReadArrayCj(parameterValue As String)

    Dim j As Long
            
    StringToArray parameterValue, fields.arrayCj, , True
            
    For j = 1 To fields.S
    
        Cj(j) = fields.arrayCj(j)
        
    Next j

End Sub

'
'   read whole Constraint type array
'
Private Sub ReadArrayConstraintType(parameterValue As String)

    Dim i As Long
            
    StringToArray parameterValue, fields.arrayConstraintType, ","
            
    For i = 1 To fields.M
    
        ConstraintType(i) = fields.arrayConstraintType(i)
        
    Next i

End Sub




'
'   read whole RHS array
'
Private Sub ReadArrayRHS(parameterValue As String)

    Dim i As Long
            
    StringToArray parameterValue, fields.arrayRHS, , True
            
    For i = 1 To fields.M
    
        RHS(i) = fields.arrayRHS(i)
        
    Next i

End Sub





'
'   read constraints structure from file
'
Private Sub ReadConstraints(fileID As Long)

    Dim parameterValue As String
    
    Dim buffer() As Variant: ReDim buffer(1 To fields.S + 2)
            
    Dim i As Long, j As Long
            
    For i = 1 To fields.M
                    
        GetParameter fileID, parameterValue
                    
        StringToArray parameterValue, buffer, , True
                
        For j = 1 To fields.S
        
            A(i, j) = Val(buffer(j))
        
        Next j
        
        ConstraintType(i) = buffer(fields.S + 1)
                
        RHS(i) = Val(buffer(fields.S + 2))
            
    Next i

End Sub


'
'   save to data definition file
'
Public Sub SaveDefinition(fileID As Long)
    
On Error GoTo errorSaveDefinition
    
    Print #fileID, "name: " & ProblemName
    
    Print #fileID, "source: " & Source
    
    Print #fileID, "comment: " & Comment
    
    Print #fileID, ModuleUtilities.PrintLine
    
    Print #fileID, "objective: " & Objective
    
    Print #fileID, "m: " & M
    
    Print #fileID, "s: " & S
    
    Print #fileID, "C(j): " & ArrayToString(fields.arrayCj, , True)
    
    Print #fileID, "constraints:"
    
    Dim i As Long
    For i = 1 To M
        
        Print #fileID, "(" & i & "): " & _
                       ArrayToString(GetMatrixRowAsArray(i, fields.matrixA), , True) & _
                       " " & ConstraintType(i) & " " & GiveFormatted(RHS(i), True)
    
    Next i
    
    Print #fileID, ModuleUtilities.PrintLine
    
    Exit Sub

errorSaveDefinition:

    Err.Raise Err.Number, , "Error saving problem definition." & _
                            NewLine(Err.Description)

End Sub

'
'   import data from other data definition file type
'
Public Sub ImportFromFile(InputFile As String)
    
On Error GoTo errorImportFromFile

    Dim fileID As Long: fileID = OpenFile(InputFile, openForInput)

    DefinitionFile = lastSaveProblemDir & "\" & ReturnFileAs(InputFile, returnNameInFile) & ".txt"

    Source = ""

    Comment = ""

    '   read MIN or MAX
    
    Dim buffer As String
    
    Do While True

        Line Input #fileID, buffer

        buffer = Left(UCase(Trim(buffer)), 3)

        If (buffer = "MIN") Or (buffer = "MAX") Then
            
            Objective = buffer
            Exit Do
            
        End If

        If EOF(fileID) Then
        
            Err.Raise vbObjectError + errorNumberBadData, , _
                      "File doesn't contain objective (MIN or MAX) declaration."
                
        End If
        
    Loop

    Dim tempArray()
    
    '   read m, s

    Line Input #fileID, buffer
    ReDim tempArray(0 To 1): StringToArray buffer, tempArray, , True
    
    M = tempArray(0)
    S = tempArray(1)

    '   read Cj coefficients
    
    Line Input #fileID, buffer: ReadArrayCj buffer
    
    '   read RHS
    
    Line Input #fileID, buffer: ReadArrayRHS buffer

    '   read matrix A

    ReDim tempArray(1 To fields.S)
    
    For i = 1 To fields.M

        Line Input #fileID, buffer: StringToArray buffer, tempArray, , True
        
        For j = 1 To fields.S
        
            A(i, j) = Val(tempArray(j))
            
        Next j
        
    Next i

    '   read the type of constraint
    
    Line Input #fileID, buffer
    
    ReadArrayConstraintType buffer
    
    Close #fileID
    
    Exit Sub

errorImportFromFile:

    If fileID > 0 Then Close #fileID
    
    ResetToDefault ProblemName
    
    Err.Raise Err.Number, , "Import definition form file." & _
                            NewLine(Err.Description)

End Sub

'
'   cell value changed
'
Private Sub DisplayDefinition_CellValueChanged(editedRow As Long, editedCol As Long, newValue As Variant)

On Error GoTo errorDisplayDefinition
    
    Dim fieldBeingEdited As String
    
    If editedRow > fields.M Then
    
        fieldBeingEdited = "Cj(" & editedCol & "): "
        Cj(editedCol) = CDbl(newValue)
        
    Else
    
        Select Case editedCol
        
        Case fields.S + 1
        
            fieldBeingEdited = "ConstraintType(" & editedRow & ") [<, =, >]: "
            ConstraintType(editedRow) = newValue
        
        Case fields.S + 2
        
            fieldBeingEdited = "RHS (" & editedRow & "): "
            RHS(editedRow) = CDbl(newValue)
        
        Case Else
        
            fieldBeingEdited = "matrix A(" & editedRow & ", " & editedCol & "): "
            A(editedRow, editedCol) = CDbl(newValue)
    
        End Select
    
    End If
    
    ProblemSaved = False
    
    Exit Sub
    
errorDisplayDefinition:

    MsgBox "Wrong input data for " & fieldBeingEdited & newValue & _
           NewLine(Err.Description), , _
           "EDIT DEFINITION ERROR!"

    DisplayDefinition.StartEditing editedRow, editedCol
    
    DisplayDefinition.editError = True

End Sub



