VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "ClassProblemResult"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False

'   CLASS MODULE: PROBLEM RESULT

'   fields containing object data

Private Type FieldType

    parentProblem As ClassProblem

    method As EnumMethod

    Status As EnumStatus

    StartedWhen As Date

    DetailFile As String

    FunctionValue As Double

    arrayBVS() As Double

    arrayXj() As Double
    
    arrayConstraintTest() As Boolean

'''    TotalIterations As Long
    
    Counters As New ClassCounters
    
    TestResult As Boolean

    Degeneracy As Boolean

    OutputDetailFile As ClassOutputDetailFile

End Type

Private fields As FieldType

'
'   counters
'
Public Property Get Counters() As ClassCounters

    Set Counters = fields.Counters
End Property

'
'   property get noDisplay
'
Public Property Get NoDisplay() As Boolean

    NoDisplay = fields.parentProblem.NoDisplay

End Property

'
'   return BVS as array
'
Public Property Get BVSAsArray()

    BVSAsArray = fields.arrayBVS

End Property

'
'   property get problem saved status
'
Public Property Get ProblemSaved() As Boolean

    ProblemSaved = fields.parentProblem.ProblemSaved

End Property

'
'   property let problem saved status
'
Public Property Let ProblemSaved(newValue As Boolean)

    fields.parentProblem.ProblemSaved = newValue

End Property

'
'   property get Definition
'
Public Property Get Definition() As ClassProblemDefinition

    Set Definition = fields.parentProblem.Definition

End Property

'   property get method name as string
'
Public Property Get MethodName() As String

    MethodName = EnumMethodAsString(fields.method)

End Property

'
'   property get Started When
'
Public Property Get StartedWhen(Optional returnFormat As String = "yyyy-mm-dd hh:mm:ss") _
                                As String
    
    StartedWhen = Format(fields.StartedWhen, returnFormat)

End Property

'
'   property let Started When
'
Public Property Let StartedWhen(Optional returnFormat As String = "yyyy-mm-dd hh:mm:ss", _
                                newValue As String)

    If Not IsDate(newValue) Then
    
        Err.Raise vbObjectError + errorNumberBadData, , _
                  "Parameter is not date: " & newValue
    
    End If

    fields.StartedWhen = CDate(newValue)

End Property

'
'   property get BVS(i)
'
Public Property Get BVS(ByVal i As Long) As Double

    BVS = fields.arrayBVS(i)

End Property

'
'   property let BVS(i)
'
Public Property Let BVS(ByVal i As Long, _
                        newValue As Double)

    If Not IsNumeric(newValue) Then
    
        Err.Raise vbObjectError + errorNumberBadData, , _
                  "BVS(" & i & ") is not numeric: " & newValue
        
    End If

    fields.arrayBVS(i) = newValue
    
End Property

'
'   property let ConstraintTest(i)
'
Public Property Let ConstraintTest(ByVal i As Long, _
                                   newValue As Boolean)

    fields.arrayConstraintTest(i) = newValue
    
End Property

'
'   property get ConstraintTest(i)
'
Public Property Get ConstraintTest(ByVal i As Long) As Boolean

    ConstraintTest = fields.arrayConstraintTest(i)
    
End Property


'
'   property get X(j)
'
Public Property Get Xj(ByVal j As Long) As Double
    
    Xj = fields.arrayXj(j)

End Property

'
'   property let X(j)
'
Public Property Let Xj(ByVal j As Long, _
                       newValue As Double)

    If Not IsNumeric(newValue) Then
    
        Err.Raise vbObjectError + errorNumberBadData, , _
                  "X(" & j & ") is not numeric: " & newValue
        
    End If
    
    fields.arrayXj(j) = newValue

End Property

'
'   property get Definition folder
'
Public Property Get DefinitionFolder() As String
    
    DefinitionFolder = ReturnFileAs(Definition.DefinitionFile, returnFilePath)

End Property

'
'   property get Solution folder
'
Public Property Get SolutionFolder() As String
    
    SolutionFolder = ReturnFileAs(Definition.DefinitionFile, returnFilePath) & "Solutions\"

End Property


'
'   property get Detail file
'
Public Property Get DetailFile(Optional getFileAs As _
                                        EnumReturnFileNameStyle = returnFileFull) _
                               As String
    
    DetailFile = ReturnFileAs(fields.DetailFile, getFileAs)

End Property


'
' property Detail file
' let
Public Property Let DetailFile(Optional getFileAs As _
                                        EnumReturnFileNameStyle = returnFileFull, _
                               newValue As String)

    If Dir(newValue) <> "" Then
    
        fields.DetailFile = newValue

    Else
    
        Status = NotCalculated
        
        fields.DetailFile = SolutionFolder & _
            Definition.DefinitionFile(returnNameInFile) & _
            ModuleUtilities.DetailFileExtension(fields.method)
    
    End If

End Property

'
'   property get Status
'
Public Property Get Status() As EnumStatus

    Status = fields.Status

End Property

'
'   property let Status
'
Public Property Let Status(newValue As EnumStatus)

    fields.Status = newValue

End Property

'
'   property get FunctionValue
'
Public Property Get FunctionValue() As Double

    FunctionValue = fields.FunctionValue

End Property

'
'   property let FunctionValue
'
Public Property Let FunctionValue(newValue As Double)

    If Not IsNumeric(newValue) Then
    
        Err.Raise vbObjectError + errorNumberBadData, , _
                  "Objective Function Value is not numeric: " & newValue
        
    End If
    
    fields.FunctionValue = newValue

End Property

'
'   property get TestResult
'
Public Property Get TestResult() As String

    TestResult = IIf(fields.TestResult, "OK", "FAILED")

End Property

'
'   property let TestResult
'
Public Property Let TestResult(newValue As String)

    newValue = Trim(newValue)

    If InStr(1, "OK FAILED", newValue) = 0 Then
    
        Err.Raise vbObjectError + errorNumberBadData, , _
                  "Unknown value for Test Result: " & newValue
        
    End If
    
    fields.TestResult = (newValue = "OK")

End Property

'
'   property get Degeneracy
'
Public Property Get Degeneracy() As String

    Degeneracy = IIf(fields.Degeneracy, "YES", "NO")

End Property

'
'   property let Degeneracy
'
Public Property Let Degeneracy(newValue As String)

    newValue = Trim(newValue)

    If InStr(1, "YES NO", newValue) = 0 Then
    
        Err.Raise vbObjectError + errorNumberBadData, , _
                  "Unknown value for Degeneracy: " & newValue
        
    End If
    
    fields.Degeneracy = (newValue = "YES")

End Property

'
'   property get OutputDetailFile
'
Public Property Get OutputDetailFile() As ClassOutputDetailFile

    Set OutputDetailFile = fields.OutputDetailFile
    
End Property

'
'   property set OutputDetailFile
'
Public Property Set OutputDetailFile(newObject As ClassOutputDetailFile)

    Set fields.OutputDetailFile = newObject
    
    fields.OutputDetailFile.InitialDeclarations Me

End Property

'
'   property get DisplayResultDetails
'
Private Property Get DisplayResultDetails() As MyRichTextBox

    Set DisplayResultDetails = fields.parentProblem.DisplayResultDetails

End Property

'
'   property get DisplayResultSummary
'
Private Property Get DisplayResultSummary() As MyGrid

    Set DisplayResultSummary = fields.parentProblem.DisplayResultSummary

End Property

'
'   property get ProblemName
'
Public Property Get ProblemName() As String
    
    ProblemName = Definition.ProblemName

End Property

'
'   property get Status as string
'
Public Property Get StatusName() As String

    StatusName = ModuleUtilities.EnumStatusAsString(fields.Status)

End Property

'
'   initial declarations of a result object
'
Public Sub InitialDeclarations(newParentProblem As ClassProblem, _
                               newMethod As EnumMethod)
                
    Set fields.parentProblem = newParentProblem
    
    fields.method = newMethod
    
    Set OutputDetailFile = New ClassOutputDetailFile
    
End Sub

'
'   set result data to default
'
Public Sub ResetToDefault()
    
    Dim i As Long, j As Long
    
    Status = NotCalculated
    
    ReDim fields.arrayBVS(1 To Definition.M)
    
    ReDim fields.arrayXj(1 To Definition.S)
    
    ReDim fields.arrayConstraintTest(1 To Definition.M)
    
    FunctionValue = 0
    
    TestResult = "FAILED"
    
    DetailFile = DefinitionFolder & ProblemName & _
                 ModuleUtilities.DetailFileExtension(fields.method)
    
    StartedWhen = Now
    
    'TotalIterations = 0
    Counters.Reset
    
    Degeneracy = "NO"

    If Not fields.parentProblem.NoDisplay Then
    
        DisplayResultSummary.ClearColumnValues DisplayResultSummary.ValueColCount
    
    End If
    
    ShowResult

End Sub

'
'   show result data (summary and detail) if exists, or clear if NotCalculated
'
Public Sub ShowResult()
    
    If NoDisplay Then Exit Sub
    
On Error GoTo errorShowResult
    
    DisplayResultSummary.ClearColumnValues fields.method
        
    DisplayResultDetails.Clear fields.method
    
    If Status > NotCalculated Then
    
        With DisplayResultSummary
            
            .SetValueSingle StatusName, 1, fields.method
            .SetValueSingle GiveFormatted(FunctionValue), , fields.method
            .SetValueSingle Degeneracy, , fields.method
            .SetValueSingle GiveFormatted(Counters(cntIterations)), , fields.method
            .SetValueSingle GiveFormatted(Counters(cntAddSubs)), , fields.method
            .SetValueSingle GiveFormatted(Counters(cntMultDivs)), , fields.method
            .SetValueSingle GiveFormatted(Counters(cntLoops)), , fields.method
            .SetValueSingle GiveFormatted(Counters(cntDecis)), , fields.method
            .SetValueSingle TestResult, , fields.method
'            .SetValueSingle DetailFile(returnFileName), , fields.method
'            .SetValueSingle StartedWhen("yyyy-mm-dd"), , fields.method
'            .SetValueSingle StartedWhen("hh:mm:ss"), , fields.method
            
            Dim i As Long
        
            For i = 1 To Definition.M
                .SetValueSingle GiveFormatted(BVS(i)), , fields.method
            Next i
            
            For i = 1 To Definition.S
                .SetValueSingle GiveFormatted(Xj(i)), , fields.method
            Next i
            
        End With
        
        DisplayResultDetails.LoadFile fields.method, DetailFile(returnFileFull)
        
    Else
    
        DisplayResultSummary.SetValueSingle StatusName, 1, fields.method
        
    End If
    
    Exit Sub

errorShowResult:

    Err.Raise Err.Number, , "Error Show Result (method: " & MethodName & ")." & _
                            NewLine(Err.Description)
        
End Sub

'
'   read data from file
'
Public Sub ReadResultData()
    
    ResetToDefault
    
    Dim fileID As Long: fileID = OpenFile(Definition.DefinitionFile, openForInput)
    
On Error GoTo errorReadResultData
    
    Dim methodFound As Boolean: methodFound = False
    Dim ParameterName As String
    Dim parameterValue As Variant
    
    Do While EOF(fileID) = False
        
        If Not methodFound Then
        
            If GetParameter(fileID, parameterValue) = "method" Then
            
                methodFound = (parameterValue = MethodName)
                
            End If
            
        Else
        
            ParameterName = GetParameter(fileID, parameterValue)
            
            If ModuleUtilities.StrAsEnumCounter(ParameterName) < cntUnknown Then
            
                Counters.CounterByName(ParameterName) = parameterValue
            
            Else
            
                Select Case ParameterName
                
                Case "method": methodFound = (parameterValue = MethodName)
                
                Case "status": Status = StringAsEnumStatus(parameterValue)
                
                Case "value": FunctionValue = Val(parameterValue)
                
                Case "degeneracy": Degeneracy = UCase(parameterValue)
                
                Case "bvs(i)": StringToArray parameterValue, fields.arrayBVS, , True
                        
                Case "x(j)": StringToArray parameterValue, fields.arrayXj, , True
                    
'''            Case "iterations": TotalIterations = Val(parameterValue)
                
                Case "testing": TestResult = UCase(parameterValue)
                
                Case "started": StartedWhen = parameterValue
                    
                Case "detail file": DetailFile = SolutionFolder & parameterValue
                        
                End Select
            
            End If
            
        End If
    
    Loop
    
    Close #fileID
    
    ShowResult
    
    Exit Sub

errorReadResultData:

    If fileID > 0 Then Close #fileID

    Status = UnknownStatus
    
    If Not NoDisplay Then
    
        DisplayResultSummary.ClearColumnValues fields.method
        DisplayResultSummary.SetValueSingle StatusName, 1, fields.method
        DisplayResultDetails.Clear fields.method
    
    End If
    
    Err.Raise Err.Number, , _
              "Reading result data from file." & _
              NewLine("File: " & Definition.DefinitionFile) & _
              NewLine("Method: " & MethodName) & _
              NewLine(Err.Description)

End Sub

'
'   save result data to file
'
Public Sub SaveResultData(fileID As Long)
    
On Error GoTo errorSaveResultData
    
    If Status <= NotCalculated Then Exit Sub
    
    '   handle new file (problem) name
        
    Dim tempDetailFile As String
    
    tempDetailFile = SolutionFolder & ProblemName & _
                     ModuleUtilities.DetailFileExtension(fields.method)
    
    If tempDetailFile <> DetailFile Then
        
        FileCopy DetailFile, tempDetailFile
        
        DetailFile = tempDetailFile
    
    End If

    '   write values to file
    
    Print #fileID, "method: " & MethodName
    
    Print #fileID, "status: " & StatusName
    
    Print #fileID, "value: " & GiveFormatted(FunctionValue, True)
    
    Print #fileID, "bvs(i): " & ArrayToString(fields.arrayBVS, , True)
    
    Print #fileID, "x(j): " & ArrayToString(fields.arrayXj, , True)
    
    Print #fileID, "degeneracy: " & Degeneracy
    
    '   counters
    Print #fileID, Counters.FileOutputStr(cntIterations)
    Print #fileID, Counters.FileOutputStr(cntAddSubs)
    Print #fileID, Counters.FileOutputStr(cntMultDivs)
    Print #fileID, Counters.FileOutputStr(cntLoops)
    Print #fileID, Counters.FileOutputStr(cntDecis)
    
    Print #fileID, "testing: " & TestResult
    
    Print #fileID, "started: " & StartedWhen
    
    Print #fileID, "detail file: " & DetailFile(returnFileName)
    
    Print #fileID, ModuleUtilities.PrintLine
    
    Exit Sub
    
errorSaveResultData:

    Status = UnknownStatus
    
    Err.Raise Err.Number, , _
              "Saving result data from file." & _
              NewLine("File: " & inputFileName) & _
              NewLine("Method: " & MethodName) & _
              NewLine(Err.Description)

End Sub

'
'   Initialization
'
Private Sub Class_Initialize()

'   initialization
End Sub

'
'   Terminate
'
Private Sub Class_Terminate()

    Set fields.Counters = Nothing
    Set fields.OutputDetailFile = Nothing
    Set fields.parentProblem = Nothing

'   termination
End Sub

