Attribute VB_Name = "ModuleGauss"

' GAUSS PIVOTING AND OTHER SUPPORT FUNCTIONS MODULE

Public Const eta As Double = 0.0000001  ' error margine

Private Const maximumIterationsAllowed = 100    ' safety reasons

' pivot given tableau using gauss pivoting rules

Public Sub GaussPivot(Inc As ClassCounters, tableau, _
                      k As Long, _
                      r As Long, _
                      BVS)
    
    Dim i As Long
    Dim j As Long
    Dim keyA As Double
    
    ' key coefficient for pivoting
    keyA = tableau(r, k)
    
    ' new Basic Vector
    BVS(r) = k
    
    ' row r
    Inc.Loops UBound(tableau, 2) - LBound(tableau, 2) + 1
    Inc.MulDiv UBound(tableau, 2) - LBound(tableau, 2) + 1
    For j = LBound(tableau, 2) To UBound(tableau, 2)
        
        tableau(r, j) = tableau(r, j) / keyA
    
    Next j
    
    ' rows other than r
    For i = LBound(tableau, 1) To UBound(tableau, 1)
        
        Inc.Decis
        If i = r Then i = i + 1: Inc.AddSub
        
        For j = LBound(tableau, 2) To UBound(tableau, 2)
            
            Inc.Decis
            If j = k Then j = j + 1: Inc.AddSub
            
            tableau(i, j) = tableau(i, j) - tableau(i, k) * tableau(r, j)
            Inc.MulDiv
            Inc.AddSub
        
            Inc.Loops
        Next j
        
        tableau(i, k) = 0
    
    Next i

End Sub

' column ratio test

Public Function ColumnRatioTest(Inc As ClassCounters, tableau, _
                                k As Long, _
                                ByRef degeneracyOccured As Boolean, _
                                Optional ignoreDegeneracy As Boolean = False) As Long
    
    Dim minimalColumnRatio As Double: minimalColumnRatio = -1
    Dim tempColumnRatio As Double
    
    Dim i As Long, j As Long
    
    Dim potencialRs() As Long
        
    Dim tempR As Long: tempR = 0
        
    Dim tempDegeneracy As Boolean: tempDegeneracy = False
      
    ReDim potencialRs(1 To 1)
    potencialRs(1) = 0
    
    ColumnRatioTest = 0
    
    For i = 1 To UBound(tableau, 1) - 1 '= m
        
        ' both RHS(k) and A(i,k) MUST be positive ?????
        Inc.Decis
        If ((tableau(i, k) > 0) And (tableau(i, UBound(tableau, 2)) >= 0)) Then
'       If ((tableau(i, k) > 0) And (tableau(i, UBound(tableau, 2)) > 0)) Then
            
            tempColumnRatio = tableau(i, UBound(tableau, 2)) / tableau(i, k)
            Inc.MulDiv
            
            ' posible degeneracy occurs
            Inc.Decis
            If tempColumnRatio = minimalColumnRatio Then
                
                tempDegeneracy = True
                
                ReDim Preserve potencialRs(1 To UBound(potencialRs) + 1)
                potencialRs(UBound(potencialRs)) = i
            
            End If
            
            Inc.Decis
            If ((tempColumnRatio < minimalColumnRatio) Or (minimalColumnRatio = -1)) Then
                
                tempDegeneracy = False
                
                ReDim potencialRs(1 To 1)
                potencialRs(1) = i
                
                minimalColumnRatio = tempColumnRatio
            
            End If
        End If
    Next i
    Inc.Loops UBound(tableau, 1)
    
    '   degeneracy handling off: break ties arbitrarily (last in)
    If ignoreDegeneracy Then
    
        degeneracyOccured = False
        ColumnRatioTest = potencialRs(1)
        Exit Function
    End If
    
    
    ' handle possible degeneracy
    Inc.Decis
    If tempDegeneracy Then
        
        j = 0
        
        Do While tempR = 0
        
            j = j + 1
            Inc.AddSub
            
            ' skip k - column
            Inc.Decis
            If j = k Then j = j + 1: Inc.AddSub
            
            ' check boundaries of tableau
            Inc.Decis
            If j > UBound(tableau, 2) - 1 Then Exit Do
            
            minimalColumnRatio = -1
            
            For i = LBound(potencialRs) To UBound(potencialRs)
                Inc.Loops
                
                ' both A(i,k) and A(i, j) MUST be positive ?????????????????
                Inc.Decis
                If ((tableau(potencialRs(i), k) > 0) And (tableau(potencialRs(i), j)) >= 0) Then
'               If ((tableau(potencialRs(i), k) > 0) And (tableau(potencialRs(i), j)) > 0) Then
                    
                    tempColumnRatio = tableau(potencialRs(i), j) / tableau(potencialRs(i), k)
                    Inc.MulDiv
                    
                    If tempColumnRatio = minimalColumnRatio Then tempR = 0
                    
                    Inc.Decis
                    If ((tempColumnRatio < minimalColumnRatio) Or (minimalColumnRatio = -1)) Then
                        
                        tempR = potencialRs(i)
                        
                        minimalColumnRatio = tempColumnRatio
                    
                    End If
                End If
            Next i
        Loop
    Else
        
        tempR = potencialRs(1)
    End If
    
    degeneracyOccured = tempDegeneracy
    
    ColumnRatioTest = tempR

End Function

' row ratio test (dual simplex)

Public Sub RowRatioTest(Inc As ClassCounters, tableau, _
                        ByRef k As Long, _
                        ByRef r As Long)
    
    Dim i As Long, j As Long
    
    Dim minimalRHS As Double: minimalRHS = 0
    
    Dim maximalRowRatio As Double
    Dim tempRowRatio As Double
    
    Dim tempR() As Long
    
    k = 0
    r = 0
    
    ' build potencial r-values (theta)
    For i = 1 To UBound(tableau, 1) - 1 '=m
        Inc.Loops
        
        ' RHS is negative
        Inc.Decis
        If tableau(i, UBound(tableau, 2)) < 0 Then
            
            ' min RHS already exists
            Inc.Decis
            If minimalRHS = tableau(i, UBound(tableau, 2)) Then
            
                ReDim Preserve tempR(UBound(tempR) + 1)
                tempR(UBound(tempR)) = i
            
            End If
            
            ' new min RHS
            Inc.Decis
            If tableau(i, UBound(tableau, 2)) < minimalRHS Then
                
                ReDim tempR(0)
                tempR(0) = i
                
                minimalRHS = tableau(i, UBound(tableau, 2))
            
            End If
        End If
    Next i
    
    ' find best RR value
    
    maximalRowRatio = 0
    
    For i = LBound(tempR) To UBound(tempR)
        
        For j = 1 To UBound(tableau, 2) - 1
            Inc.Loops
                        
            Inc.Decis
            If tableau(UBound(tableau, 1), j) < 0 Then  ' Cj has to be negative
                
                Inc.Decis
                If tableau(tempR(i), j) < 0 Then ' A(i,j) has to be negative
                    
                    tempRowRatio = tableau(UBound(tableau, 1), j) / tableau(tempR(i), j)
                    Inc.MulDiv
                    
                    Inc.Decis
                    If maximalRowRatio < tempRowRatio Then
                    
                        maximalRowRatio = tempRowRatio
                        r = tempR(i)
                        k = j
                    End If
                End If
            End If
        Next j
    Next i
End Sub

' increase iteration count and check if too big

Public Function IterationCountExceedsMaxAllowed(Inc As ClassCounters) As Boolean

    Inc.Iters
    
    IterationCountExceedsMaxAllowed = (Inc.Counter(cntIterations) > maximumIterationsAllowed)
End Function



' calculate optimal solution

Public Sub CalculateOptimalSolution(Result As ClassProblemResult, _
                                    Definition As ClassProblemDefinition, _
                                    tableau, _
                                    BVS)
    
    Dim i As Long, j As Long
    Dim trivialSolution As Boolean: trivialSolution = True
    
    ' calculate values
    For j = 1 To Definition.S
    
        Result.Xj(j) = 0
    
    Next j
    
    Result.FunctionValue = 0
    
    For i = 1 To Definition.M
        
        Result.BVS(i) = BVS(i)
        
        If (BVS(i) > 0) And (BVS(i) <= Definition.S) Then
        
            Result.Xj(BVS(i)) = tableau(i, UBound(tableau, 2))
            
            trivialSolution = IIf(Result.Xj(BVS(i)) <> 0, False, trivialSolution)
            
            Result.FunctionValue = Result.FunctionValue + _
                Result.Xj(BVS(i)) * Definition.Cj(BVS(i))
            
        End If
    
    Next i
    
    Result.TestResult = "FAILED"
    
    '   testing for conditions
    Dim tempValue As Double
    Dim tempTest As Boolean: tempTest = True
    For i = 1 To Definition.M
        
        tempValue = 0
        
        For j = 1 To Definition.S
            
            tempValue = tempValue + Definition.A(i, j) * Result.Xj(j)
        
        Next j
        
        Result.ConstraintTest(i) = False
        
        Select Case Definition.ConstraintType(i)
            
        Case "="
            
            If Abs(tempValue - Definition.RHS(i)) <= eta Then
            
                Result.ConstraintTest(i) = True
                
            End If
            
        Case ">"
        
            If tempValue >= Definition.RHS(i) - eta Then
            
                Result.ConstraintTest(i) = True
                
            End If
            
        Case "<"
        
            If tempValue <= Definition.RHS(i) + eta Then
            
                Result.ConstraintTest(i) = True
                
            End If
        
        End Select
        
        tempTest = (tempTest And Result.ConstraintTest(i))
        
    Next i
    
    Result.TestResult = IIf(tempTest, "OK", "FAILED")
    
    '   recalculate status
    If trivialSolution Then Result.Status = Trivial
    If Result.TestResult = "FAILED" Then Result.Status = TestFailed
    
'''    2004-12-05 recalculating status
'''    '   test for trivial solution
'''    If Not trivialSolution Then
'''        '   not all x(j) are 0
'''        If Result.TestResult = "FAILED" Then Result.Status = TestFailed
'''
'''    Else
'''        '   the solution is trivial
'''        Result.Status = Trivial
'''
'''    End If
    
End Sub


