Attribute VB_Name = "ModuleUtilities"

'   MODULE: UTILITY CONSTANTS, ENUMERATORS, SUBS AND FUNCTIONS

'   error number constants: ClassProblemDefinition

Public Const errorNumberDefinitionFile = 2000       ' handling definition file error
Public Const errorNumberBadData = 2001              ' bad data input (reading data)

'   error number constants: ClassProblemResult

Public Const errorNumberDetailFile = 3000           ' detail file error
Public Const errorNumberDefinitionAppend = 3001     ' append to definition file error
Public Const errorNumberReadResultData = 3002       ' open read result file error

'   error number constants: ModuleUtilities

Public Const errorNumberStringToArray = 4000        ' convert string to array error
Public Const errorNumberOpenFile = 4001             ' error opening file

'   enumerating method

Public Enum EnumMethod
    
    UnknownMethod = 0
    PushPull = 1
    Simplex = 2

End Enum

'   enumerating result statuses

Public Enum EnumStatus
    
    UnknownStatus = -1
    NotCalculated
    Optimal
    Infeasible
    Unbounded
    MaxIterationsExceeded
    TestFailed
    Trivial

End Enum

'   enumerating file name return style

Public Enum EnumReturnFileNameStyle
    
    returnFileFull
    returnFilePath
    returnFileName
    returnFileExtension
    returnNameInFile

End Enum

'   enumerating open file modifiers

Public Enum EnumOpenFileAccess
    
    openForInput
    openForOutput
    openForAppend

End Enum

'   enumerating counters

Public Enum EnumCounter
    
    cntIterations = 0
    cntAddSubs = 1
    cntMultDivs = 2
    cntLoops = 3
    cntDecis = 4

    cntUnknown = 9
End Enum

'
' return string representation for counter type
'
Public Function EnumCounterAsStr(EnumCntCode As EnumCounter, _
                                    Optional Short As Boolean = False) As String
    
    Select Case EnumCntCode
    
    Case EnumCounter.cntIterations: EnumCounterAsStr = IIf(Short, "Iter.", "iterations")
    Case EnumCounter.cntAddSubs: EnumCounterAsStr = IIf(Short, "+/-", "adds/subs")
    Case EnumCounter.cntMultDivs: EnumCounterAsStr = IIf(Short, "*/", "mults/divs")
    Case EnumCounter.cntLoops: EnumCounterAsStr = IIf(Short, "Loop", "loops")
    Case EnumCounter.cntDecis: EnumCounterAsStr = IIf(Short, "Dec.", "decisions")
    Case Else: EnumCounterAsStr = IIf(Short, "Unknown", "unknown counter")
    
    End Select

End Function

'
' convert string to enum counter type
'
Public Function StrAsEnumCounter(EnumString As String) As EnumCounter
    
    Select Case EnumString
    
    Case EnumCounterAsStr(cntIterations, True), EnumCounterAsStr(cntIterations)
        StrAsEnumCounter = cntIterations
    Case EnumCounterAsStr(cntAddSubs, True), EnumCounterAsStr(cntAddSubs)
        StrAsEnumCounter = cntAddSubs
    Case EnumCounterAsStr(cntMultDivs, True), EnumCounterAsStr(cntMultDivs)
        StrAsEnumCounter = cntMultDivs
    Case EnumCounterAsStr(cntLoops, True), EnumCounterAsStr(cntLoops)
        StrAsEnumCounter = cntLoops
    Case EnumCounterAsStr(cntDecis, True), EnumCounterAsStr(cntDecis)
        StrAsEnumCounter = cntDecis
    Case Else: StrAsEnumCounter = cntUnknown
    
    End Select

End Function

'
'   make an exact duplicate of an array
'
Public Function CloneArray(sourceArray)
                      
    Dim newCloneArray()
    
    ReDim newCloneArray(LBound(sourceArray, 1) To UBound(sourceArray, 1))
        
    Dim i As Long
    
    For i = LBound(sourceArray, 1) To UBound(sourceArray, 1)
        
        newCloneArray(i) = sourceArray(i)
        
    Next i
        
    CloneArray = newCloneArray
        
End Function

'
'   copy contetns of one array to another
'
Public Sub CopyArray(sourceArray, _
                     targetArray, _
                     defaultValue, _
                     Optional preserveValues As Boolean = True)
                      
    Dim i As Long
                      
    For i = LBound(targetArray, 1) To UBound(targetArray, 1)
        
        If (i >= LBound(sourceArray, 1)) And _
           (i <= UBound(sourceArray, 1)) And _
           (preserveValues) Then
                
'            CopyValue targetArray(i), sourceArray(i)
            GetFormatted targetArray(i), sourceArray(i)
                
        Else
            
            targetArray(i) = defaultValue
            
        End If
        
    Next i

End Sub


'
'   copy values from one matrix to another
'
Public Sub CopyMatrix(sourceMatrix, _
                      targetMatrix, _
                      defaultValue, _
                      Optional preserveValues As Boolean = True)
                      
    Dim i As Long, j As Long
    
    For i = LBound(targetMatrix, 1) To UBound(targetMatrix, 1)
            
        For j = LBound(targetMatrix, 2) To UBound(targetMatrix, 2)
        
            If ((i >= LBound(sourceMatrix, 1)) And (i <= UBound(sourceMatrix, 1))) And _
               ((j >= LBound(sourceMatrix, 2)) And (j <= UBound(sourceMatrix, 2))) And _
               (preserveValues) Then
                
'                CopyValue targetMatrix(i, j), sourceMatrix(i, j)
                GetFormatted targetMatrix(i, j), sourceMatrix(i, j)
                
                    
            Else
                
                targetMatrix(i, j) = defaultValue
                
            End If
                
        Next j
        
    Next i

End Sub

'
'   create exact copy of a matrix
'
Public Function CloneMatrix(sourceMatrix)

    Dim newCloneMatrix()
                       
    ReDim newCloneMatrix(LBound(sourceMatrix, 1) To UBound(sourceMatrix, 1), _
                         LBound(sourceMatrix, 2) To UBound(sourceMatrix, 2))
        
    Dim i As Long, j As Long
    
    For i = LBound(sourceMatrix, 1) To UBound(sourceMatrix, 1)
            
        For j = LBound(sourceMatrix, 2) To UBound(sourceMatrix, 2)
        
            newCloneMatrix(i, j) = sourceMatrix(i, j)
                
        Next j
        
    Next i
        
    CloneMatrix = newCloneMatrix
        
End Function

'
'   controled copying of values: mostly because of confusion with
'   international use of character used with decimal numbers (usually char . or char ,)
'
Public Sub CopyValue(var1, var2)

On Error GoTo errorCopyValue

    Select Case TypeName(var1)
    
    Case "Double", "Single", "Decimal", "Currency"
        
        var1 = Val(var2)

    Case Else
    
        var1 = var2
        
    End Select
    
    Exit Sub
    
errorCopyValue:

    Err.Raise Err.Number, , Err.Description & _
              NewLine("Value assignment error: " & _
              var1 & "(" & TypeName(var1) & ") to " & _
              var2 & "(" & TypeName(var2) & ")")

End Sub

'
'   create array based on martix row
'
Public Function GetMatrixRowAsArray(rowIndex As Long, _
                                    sourceMatrix)

    Dim newRowAsArray()
                       
    ReDim newRowAsArray(LBound(sourceMatrix, 2) To UBound(sourceMatrix, 2))
        
    Dim j As Long
    
    For j = LBound(sourceMatrix, 2) To UBound(sourceMatrix, 2)
        
        newRowAsArray(j) = sourceMatrix(rowIndex, j)
                
    Next j
        
    GetMatrixRowAsArray = newRowAsArray
        
End Function

'
'   create array based on martix column
'
Public Function GetMatrixColumnAsArray(columnIndex As Long, _
                                       sourceMatrix)

    Dim newColumnAsArray()
                       
    ReDim newColumnAsArray(LBound(sourceMatrix, 1) To UBound(sourceMatrix, 1))
        
    Dim i As Long
    
    For i = LBound(sourceMatrix, 1) To UBound(sourceMatrix, 1)
        
        newColumnAsArray(i) = sourceMatrix(i, columnIndex)
                
    Next i
        
    GetMatrixColumnAsArray = newColumnAsArray
        
End Function


'
'   read next line from file, return parameter name and value
'
Public Function GetParameter(fileID As Long, _
                             ByRef parameterValue) As String
    
On Error GoTo errorGetParameter
    
    GetParameter = ""
    
    Dim buffer As String: buffer = "unread line"
    
    Dim p As Long
    
    Line Input #fileID, buffer
        
    p = InStr(1, buffer, ":")
        
    If p > 0 Then
    
        GetParameter = Trim(LCase(Left(buffer, p - 1)))
        
        parameterValue = Trim(Mid(buffer, p + 1))
        
    End If

    Exit Function
    
errorGetParameter:

    Err.Raise Err.Number, , "Error reading parameter." & _
                            Err.Description & _
                            NewLine("Line: " & buffer)

End Function

'
'   simple test if file is problem definition file
'
Public Function IsProblemDefinition(fullFileName As String) As Boolean

On Error GoTo errorIsProblemDefinition

    Dim fileID As Long: fileID = OpenFile(fullFileName, openForInput)
    
    Dim buffer As String: Line Input #fileID, buffer
    
    IsProblemDefinition = (Left(Trim(buffer), 5) = "name:")

    Close #fileID

    Exit Function

errorIsProblemDefinition:

    If fileID > 0 Then Close #fileID

    Err.Raise Err.Number, Err.Source, "Testing for Problem DefinitionFile", _
                            NewLine("File: " & fullFileName), _
                            NewLine(Err.Description)

End Function

'
'   return string with capital first characters
'
Public Function PascalType(original As String) As String

    PascalType = ""
    
    Dim i As Long, SpaceDetected As Boolean: SpaceDetected = True
    For i = 1 To Len(original)
    
        If SpaceDetected Then
        
            PascalType = PascalType & UCase(Mid(original, i, 1))
            SpaceDetected = False
            
        Else
        
            PascalType = PascalType & LCase(Mid(original, i, 1))
            
        End If
        
        SpaceDetected = (Mid(original, i, 1) = " ")
    
    Next i

End Function

'
'   redimension array, keep old values add default if new is larger
'
Public Sub ReDimArray(inputArray, _
                      newLowerBound As Long, _
                      newUpperBound As Long, _
                      defaultValue, _
                      Optional preserveValues As Boolean = True)

    If newUpperBound < newLowerBound Then Exit Sub
    
    Dim buffer()
    
    buffer = CloneArray(inputArray)
    
    ReDim inputArray(newLowerBound To newUpperBound)
    
    CopyArray buffer, inputArray, defaultValue, preserveValues

End Sub

'
'   redimension array, keep old values add default if new is larger
'
Public Sub ReDimMatrix(inputMatrix, _
                       newRowLowerBound As Long, _
                       newRowUpperBound As Long, _
                       newColLowerBound As Long, _
                       newColUpperBound As Long, _
                       defaultValue, _
                       Optional preserveValues As Boolean = True)

    If newRowUpperBound < newRowLowerBound Then Exit Sub
    
    If newColUpperBound < newColLowerBound Then Exit Sub
    
    Dim buffer()
    
    buffer = CloneMatrix(inputMatrix)
    
    ReDim inputMatrix(newRowLowerBound To newRowUpperBound, _
                      newColLowerBound To newColUpperBound)
    
    CopyMatrix buffer, inputMatrix, defaultValue, preserveValues

End Sub

'
'   test if item exists in collection
'
Public Function ExistsInCollection(objectCollection As Collection, _
                                   objectKey, _
                                   Optional returnObject As Object) _
                                   As Boolean

On Error GoTo errorExistsInCollection

    ExistsInCollection = True
    
    Set returnObject = objectCollection(objectKey)

    Exit Function

errorExistsInCollection:

    ExistsInCollection = False

End Function


'
'   merge a par of arrays
'
Public Function MergeArrays(leftArray, _
                            rightArray)

    Dim buffer()
    
    ReDim buffer(LBound(leftArray) To UBound(leftArray) + (UBound(rightArray) - LBound(rightArray)) + 1)
    
    Dim i As Long, j As Long
    
    j = LBound(leftArray)
    
    For i = LBound(buffer) To UBound(buffer)
    
        If i = UBound(leftArray) + 1 Then j = LBound(rightArray)
        
        If i <= UBound(leftArray) Then
        
            buffer(i) = leftArray(j)
            
        Else
        
            buffer(i) = rightArray(j)
        
        End If
    
        j = j + 1
    
    Next i
    
    MergeArrays = buffer

End Function


'
'   fill matrix row from given array
'
Public Sub SetMatrixRowFromArray(rowIndex As Long, _
                                 sourceArray, _
                                 targetMatrix)

    Dim j As Long
    
    For j = LBound(targetMatrix, 2) To UBound(targetMatrix, 2)
            
        GetFormatted targetMatrix(rowIndex, j), sourceArray(j), True
    
    Next j
        
    Exit Sub
        
End Sub

'
'   return desired part of array as array
'
Public Function PartOfArray(sourceArray, _
                            startIndex As Long, _
                            endIndex As Long)

    Dim resultArray(): ReDim resultArray(1 To (endIndex - startIndex + 1))
    
    Dim j As Long
    
    For j = LBound(resultArray) To UBound(resultArray)
        
        resultArray(j) = sourceArray(startIndex + j - 1)
    
    Next j
        
    PartOfArray = resultArray
        
End Function


'
'   return string with CRLF at the beggining
'
Public Function NewLine(newLineText As String) As String

    NewLine = Chr(13) & Chr(10) & newLineText

End Function

'
'   return string of characters "-"
'
Public Function PrintLine(Optional lineLen = 50) As String

    PrintLine = String(lineLen, "-")

End Function

'
'   return string containing given parameter with spaces on left or right
'
Public Function Spaces(inString, _
                       length As Long, _
                       Optional position = "> | <") As String
                       
    Dim buffer As String: buffer = inString
    
    buffer = Trim(buffer)
    
    position = Left(position, 1)
    
    If Len(buffer) >= length Then
    
        Spaces = buffer
        
    Else
    
        Select Case Left(position, 1)
        '   first char because of default value, which is showing possible values as well
        '   as giving the default value: >
        
        Case ">": Spaces = Right(Space(length) & buffer, length)
            
        Case "<": Spaces = Left(buffer & Space(length), length)
        
        Case "|"
            
            Dim divide As Long: divide = Int((length - Len(buffer)) / 2)
        
            Spaces = String(divide, ".") & buffer & String(length - Len(buffer) - divide, ".")
        
        Case Else: Spaces = buffer
        
        End Select
    
    End If
                       
End Function

'
'   fill array from string delimited with spaces
'
Public Sub StringToArray(ByVal sourceString As String, _
                         targetArray, _
                         Optional delimiter As String = " ", _
                         Optional readFromFileFormat As Boolean = False)

On Error GoTo errorStringToArray
    
    Dim i As Long
    
    Dim p As Long
    
    Dim newValue As Variant
    
    Dim buffer As String
    
    buffer = Trim(sourceString) & delimiter
    
    p = InStr(1, buffer, delimiter)
    
    i = LBound(targetArray) - 1
    
    Do While p > 0
    
        i = i + 1
        
        If i > UBound(targetArray) Then
        
            Err.Raise vbObjectError + errorNumberStringToArray, "ProjectSixPap.ModuleUtilities", _
                "Source string longer than target array." & _
                NewLine("Array length: " & UBound(targetArray) - LBound(targetArray) + 1)
        
        End If
        
        newValue = Left(buffer, p - 1)
        
        buffer = Mid(buffer, p + Len(delimiter))
        
        GetFormatted targetArray(i), newValue, True
        
        p = InStr(1, buffer, delimiter)
    
    Loop
    
    Exit Sub
    
errorStringToArray:

    Err.Raise Err.Number, , "String to Array." & _
                            NewLine("String: '" & sourceString & "'") & _
                            NewLine(Err.Description)

End Sub


                       
'
'   create space delimited string from array
'
Public Function ArrayToString(sourceArray, _
                              Optional delimiter As String = " ", _
                              Optional formatForWriteToFile As Boolean = False) _
                              As String

    
    Dim i As Long: i = LBound(sourceArray)
    
    Dim buffer As String: buffer = GiveFormatted(sourceArray(i), formatForWriteToFile)
    
    For i = LBound(sourceArray) + 1 To UBound(sourceArray)
    
        buffer = buffer & delimiter & GiveFormatted(sourceArray(i), formatForWriteToFile)
    
    Next i
    
    ArrayToString = buffer
    
End Function

'
' return string representation for enumerated method
'
Public Function EnumMethodAsString(method As EnumMethod) As String
    
    Select Case method
    
    Case PushPull: EnumMethodAsString = "PUSH AND PULL"
    Case Simplex: EnumMethodAsString = "STD. SIMPLEX"
    Case Else: EnumMethodAsString = "UNKNOWN METHOD"
    
    End Select

End Function

'
' return string representation for open access mode
'
Public Function EnumOpenAccessModeAsString(openAccessMode As EnumOpenFileAccess) As String
    
    Select Case openAccessMode
    
    Case openForAppend: EnumOpenAccessModeAsString = "Open for Append"
    
    Case openForInput: EnumOpenAccessModeAsString = "Open for Input"
    
    Case openForOutput: EnumOpenAccessModeAsString = "Open for Output"
    
    Case Else: EnumOpenAccessModeAsString = "Unknown Access"
    
    End Select

End Function

'
' return default extension for selected method
'
Public Function DetailFileExtension(method As EnumMethod) As String
    
    Select Case method
    
    Case PushPull: DetailFileExtension = ".pap"
    Case Simplex: DetailFileExtension = ".six"
    Case Else: DetailFileExtension = ".???"
    
    End Select

End Function

'
' return enumerated method representation for string
'
Public Function StringAsEnumMethod(method As String) As EnumMethod
    
    Select Case method
    
    Case EnumMethodAsString(PushPull): StringAsEnumMethod = PushPull
    Case EnumMethodAsString(Simplex): StringAsEnumMethod = Simplex
    Case Else: StringAsEnumMethod = UnknownMethod
    
    End Select

End Function

'
' returning different file name styles
'
Public Function ReturnFileAs(fullFileName As String, _
                             style As EnumReturnFileNameStyle) As String
    Dim p As Long
    
    ReturnFileAs = fullFileName
        
    Select Case style
    
    Case returnFileName
    
        p = InStrRev(fullFileName, "\")
        If p > 0 Then ReturnFileAs = Mid(fullFileName, p + 1)
    
    Case returnFilePath
    
        p = InStrRev(fullFileName, "\")
        If p > 0 Then ReturnFileAs = Left(fullFileName, p)
    
    Case returnFileExtension
        
        p = InStrRev(fullFileName, ".")
        If p > 0 Then ReturnFileAs = Mid(fullFileName, p + 1)
    
    Case returnNameInFile
    
        p = InStrRev(fullFileName, "\")
        If p > 0 Then ReturnFileAs = Mid(fullFileName, p + 1)
        p = InStrRev(ReturnFileAs, ".")
        If p > 0 Then ReturnFileAs = Left(ReturnFileAs, p - 1)
    
    End Select

End Function

'
' return string representation for enumerated status
'
Public Function EnumStatusAsString(Status As EnumStatus) As String
    
    Select Case Status
    
    Case EnumStatus.NotCalculated: EnumStatusAsString = "NOT CALCULATED YET"
    Case EnumStatus.Optimal: EnumStatusAsString = "OPTIMAL"
    Case EnumStatus.Infeasible: EnumStatusAsString = "INFEASIBLE"
    Case EnumStatus.Unbounded: EnumStatusAsString = "UNBOUNDED"
    Case EnumStatus.MaxIterationsExceeded: EnumStatusAsString = "MAX ITERATIONS EXCEEDED"
    Case EnumStatus.TestFailed: EnumStatusAsString = "TEST FAILED"
    Case EnumStatus.Trivial: EnumStatusAsString = "TRIVIAL"
    Case Else: EnumStatusAsString = "UNKNOWN STATUS"
    
    End Select

End Function

'
' return enumerated status representation for string
'
Public Function StringAsEnumStatus(ByVal Status As String) As EnumStatus
    
    Select Case Status
    
    Case EnumStatusAsString(NotCalculated): StringAsEnumStatus = NotCalculated
    Case EnumStatusAsString(Optimal): StringAsEnumStatus = Optimal
    Case EnumStatusAsString(Infeasible): StringAsEnumStatus = Infeasible
    Case EnumStatusAsString(Unbounded): StringAsEnumStatus = Unbounded
    Case EnumStatusAsString(MaxIterationsExceeded): StringAsEnumStatus = MaxIterationsExceeded
    Case EnumStatusAsString(TestFailed): StringAsEnumStatus = TestFailed
    Case EnumStatusAsString(Trivial): StringAsEnumStatus = Trivial
    Case Else: StringAsEnumStatus = UnknownStatus
    
    End Select

End Function

'
' open file for i/o operation, handle errors
'
Public Function OpenFile(fullFileName As String, _
                         openFileAccess As EnumOpenFileAccess) As Long

On Error GoTo errorOpenFile

    ' override file attributes (when copied from CD, files become READ-ONLY!)
    If Dir(fullFileName) <> "" Then SetAttr fullFileName, vbNormal
    
    OpenFile = FreeFile
    
    Select Case openFileAccess
    
    Case openForInput: Open fullFileName For Input As #OpenFile
    
    Case openForOutput: Open fullFileName For Output As #OpenFile
    
    Case openForAppend: Open fullFileName For Append As #OpenFile
    
    End Select
    
    Exit Function
    
errorOpenFile:
    
    If OpenFile > 0 Then Close #OpenFile
    
    Err.Raise vbObjectError + errorNumberOpenFile, Err.Source, _
              "Open file error." & _
              NewLine("Access mode: " & EnumOpenAccessModeAsString(openFileAccess)) & _
              NewLine("File: " & fullFileName)
    
    OpenFile = -1
    
End Function

'
'   handling text format for numbers
'
Public Function GiveFormatted(givenValue, _
                              Optional formatForWriteToFile As Boolean = False) _
                              As String
    
    Dim defaultFormat As String: defaultFormat = " 0.00e+00;-0.00e+00"
    
    Dim buffer As String, p As Long
    
    If IsNull(givenValue) Then givenValue = ""
    
    If IsNumeric(givenValue) Then
        
        buffer = Str(givenValue)
        
        If formatForWriteToFile Then
        
            buffer = Trim(buffer)
        
        Else
        
            p = InStr(1, buffer, ".")
            If p > 0 Then
            
                buffer = Format(givenValue, "#,##0.####")  '& String(Len(defaultFormat), "#"))
        
            Else
            
                buffer = Format(givenValue, "#,##0")
        
            End If
        
            buffer = Trim(buffer)
        
            If Len(buffer) > (Len(defaultFormat) - 1) / 2 Then
            
                buffer = Trim(Format(givenValue, defaultFormat))
            
            End If
            
        End If
    
    Else
    
        buffer = Trim(givenValue)
    
    End If
    
    GiveFormatted = buffer

End Function

'
'   handling text format for numbers
'
Public Sub GetFormatted(receivingVariable, _
                        valueGiven As Variant, _
                        Optional readFromFileFormat As Boolean = False)
          
    Select Case TypeName(receivingVariable)
    
    Case "Double", "Single", "Currency"
    
        If readFromFileFormat Then
        
            receivingVariable = Val(valueGiven)
    
        Else
        
            receivingVariable = valueGiven
        
        End If
    
    Case Else
    
        receivingVariable = valueGiven
    
    End Select

End Sub


'
' stuff string with substring
'
Public Function Stuff(originalString As String, _
                      stringToInsert As String) As String
    
    Dim i As Long
    
    Stuff = ""
    
    For i = 1 To Len(originalString)
    
        Stuff = Stuff & Mid(originalString, i, 1) & stringToInsert
    
    Next i

End Function

'
'   determine, 0, 1 or 2, don't want to handle more than that for now
'
Public Function MatrixDimension(matrix) As Long

On Error GoTo errorMatrixDimension

    Dim tempDimension As Long, tmp As Long
    
    For tempDimension = 1 To 3
        
        tmp = UBound(matrix, tempDimension)
    
    Next

errorMatrixDimension:

    MatrixDimension = tempDimension - 1

End Function



'
'   block write combination of scalars, vectors and matrixes to a file
'
Public Sub MatrixBlockWrite(fileID As Long, _
                            delimiter As String, _
                            ParamArray matrixes())
                            
On Error GoTo errorMatrixBlockWrite
                                
    Dim matrixCount As Long
    
    Dim matrixDim() As Long
    ReDim matrixDim(LBound(matrixes) To UBound(matrixes))
    
    Dim firstRow As Long, lastRow As Long
    
    '   determine matrix dimensions
    For matrixCount = LBound(matrixes) To UBound(matrixes)
    
        matrixDim(matrixCount) = MatrixDimension(matrixes(matrixCount))
        
        If matrixDim(matrixCount) > 0 Then
        
            firstRow = LBound(matrixes(matrixCount), 1)
            lastRow = UBound(matrixes(matrixCount), 1)
        
        End If

    Next matrixCount

    '   create string and write to a file

    Dim buffer As String
    Dim i As Long, j As Long
    
    For i = firstRow To lastRow
        
        buffer = ""
        
        For matrixCount = LBound(matrixes) To UBound(matrixes)
    
            Select Case matrixDim(matrixCount)
            
            Case 0: buffer = buffer & NumToStr(matrixes(matrixCount)) & delimiter
            
            Case 1: buffer = buffer & NumToStr(matrixes(matrixCount)(i)) & delimiter
            
            Case 2
            
                For j = LBound(matrixes(matrixCount), 2) To UBound(matrixes(matrixCount), 2)
            
                    buffer = buffer & NumToStr(matrixes(matrixCount)(i, j)) & delimiter
                    
                Next j
                
            End Select
    
        Next matrixCount
        
        '   execute printing
        
        buffer = Left(buffer, Len(buffer) - Len(delimiter))
    
        Print #fileID, buffer
    
    Next i
    
    Exit Sub
    
errorMatrixBlockWrite:

    Err.Raise Err.Number, , "Trying to write several matrixes to a file." & _
                            NewLine(Err.Description)
                            
End Sub


'
'   convert parameter to string, if numeric with Str()
'
Public Function NumToStr(parameter) As String

    If IsNumeric(parameter) Then
    
        NumToStr = "0" & Trim(Str(Abs(parameter)))
        If parameter < 0 Then NumToStr = "-" & NumToStr
        
    Else
    
        NumToStr = Trim(parameter)
    
    End If

End Function

'
'   convert parameter to string, if numeric with Str()
'
Public Function NumFromStr(parameter As String)

    If IsNumeric(parameter) Then
    
        NumFromStr = Val(parameter)
        
    Else
    
        NumFromStr = Trim(parameter)
    
    End If

End Function


